"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const decodeObjectIdentifier_1 = tslib_1.__importDefault(require("../codecs/x690/decoders/decodeObjectIdentifier"));
const encodeObjectIdentifier_1 = tslib_1.__importDefault(require("../codecs/x690/encoders/encodeObjectIdentifier"));
class ObjectIdentifier {
    constructor(nodes, prefix) {
        const _nodes = prefix
            ? typeof prefix === "number"
                ? [prefix, ...nodes]
                : [...prefix._nodes, ...nodes]
            : nodes.slice(0);
        if (_nodes.length < 2) {
            throw new Error("Cannot construct an OID with less than two nodes!");
        }
        if (_nodes.length >= 1 && !(_nodes[0] in [0, 1, 2])) {
            throw new Error("OIDs first node must be 0, 1, or 2!");
        }
        if (!_nodes.every((node) => Number.isSafeInteger(node))) {
            throw new Error("OID contained a non-safe integer value.");
        }
        if (((_nodes[0] < 2) && _nodes[1] > 39)
            || (_nodes[0] === 2 && _nodes[1] > 175)) {
            throw new Error("OID Node #2 cannot exceed 39 if node #1 is 0 or 1, and 175 "
                + `if node #1 is 2. Received these nodes: ${_nodes}.`);
        }
        for (const node of _nodes) {
            if (node < 0) {
                throw new Error("OID node numbers cannot be negative!");
            }
            if (node > Number.MAX_SAFE_INTEGER) {
                throw new Error("OID number was too big!");
            }
        }
        this._nodes = _nodes;
    }
    get nodes() {
        return this._nodes.slice(0);
    }
    get dotDelimitedNotation() {
        return this._nodes.join(".");
    }
    get asn1Notation() {
        return `{ ${this._nodes.map((node) => node.toString()).join(" ")} }`;
    }
    toString() {
        return this.dotDelimitedNotation;
    }
    toJSON() {
        return this.dotDelimitedNotation;
    }
    toBytes() {
        return (0, encodeObjectIdentifier_1.default)(this);
    }
    static fromString(str) {
        return new ObjectIdentifier(str.split(".").map((arc) => Number.parseInt(arc, 10)));
    }
    static fromBytes(bytes) {
        return (0, decodeObjectIdentifier_1.default)(bytes);
    }
    static compare(a, b) {
        if (a._nodes.length !== b._nodes.length) {
            return false;
        }
        const len = a._nodes.length;
        let i = len - 1;
        while (i >= 0) {
            if (a._nodes[i] !== b._nodes[i]) {
                return false;
            }
            i--;
        }
        return true;
    }
    isEqualTo(other) {
        return ObjectIdentifier.compare(this, other);
    }
}
exports.default = ObjectIdentifier;
//# sourceMappingURL=ObjectIdentifier.js.map